const kLatestFlashVersion = [10, 1, 90, 3];
const kLatest10_3FlashVersion = [9, 0, 283, 0];

// Returns an array of Flash version components (either 3 or 4, depending on how
// much information is available).
// Returns null if the version can't be detected. Only supports browsers that
// support navigator.plugins.
function getFlashVersion() {
  if (navigator.plugins == null)
    return null;
  var flashPlugin = navigator.plugins['Shockwave Flash'];
  if (!flashPlugin)
    return null;

  // Gecko 1.9.2+ has a version property.
  if (flashPlugin.version)
    return flashPlugin.version.split('.');

  // Otherwise parse the description.
  var descriptionComponents = flashPlugin.description.split(' ');
  var versionComponents = descriptionComponents[2].split('.');
  var revision = descriptionComponents[3];
  if (revision == '')
    revision = descriptionComponents[4];

  if (revision[0] == 'd') {
    revision = revision.substring(1);
  } else if (revision[0] == 'r') {
    revision = revision.substring(1);
    var dIndex = revision.indexOf('d');
    if (dIndex > 0)
      revision = revision.substring(0, dIndex);
  }
  versionComponents.push(revision);
  return versionComponents;
}

// Returns true if the user can run the latest version of Flash.
function canRunLatestFlash() {
  return isTigerOrLater();
}

// Returns true if the user is running OS X.
function isMacOS() {
  return navigator.userAgent.indexOf('Mac OS X') != -1;
}

// Returns an array of OS X version components (either 2 or 3, depending on how
// much information is available).
// Returns null if the OS version can't be detected.
function getMacOSVersion() {
  var osPrefix = 'Mac OS X ';
  var prefixIndex = navigator.userAgent.indexOf(osPrefix);
  if (prefixIndex == -1)
    return null;
  var versionString = navigator.userAgent.substring(prefixIndex + osPrefix.length).split(';')[0];
  if (versionString.indexOf('_') != -1)
    return versionString.split('_');
  else
    return versionString.split('.');
}

// Returns true if the user is running at least 10.4.
// A false return only means that the user might be running something older,
// not that they definitely are.
function isTigerOrLater() {
  var osVersion = getMacOSVersion();
  if (osVersion)
    return osVersion[1] && (osVersion[1] >= 4);
  // If the version isn't available, check for a version of Flash that requires
  // 10.4+.
  var flashVersion = getFlashVersion();
  return flashVersion && (flashVersion[0] > 9);
}

// Takes two arrays of version components, and returns -1 if versionA is less
// than versionB, 1 if versionA is greater than versionB, and 0 if they are the
// same.
// The match is only as granular as the shorter version; e.g., comparing 10.5
// and 10.5.8 will return 0.
function compareVersions(versionA, versionB) {
  compareLength = Math.min(versionA.length, versionB.length);
  for (var i = 0; i < compareLength; i++) {
    if (versionA[i] < versionB[i])
      return -1;
    if (versionA[i] > versionB[i])
      return 1;
  }
  return 0;
}

// Returns true if the user is definitely not running the latest version of
// Flash. A false return means either they are up to date, or it's not possible
// to tell.
// Only supports OS X.
function isFlashOutOfDate() {
  if (!isMacOS())
    return false;
  var userFlashVersion = getFlashVersion();
  if (isTigerOrLater())
    return compareVersions(userFlashVersion, kLatestFlashVersion) < 0;
  return compareVersions(userFlashVersion, kLatest10_3FlashVersion);
}

// Checks to see if the user is running an out-of-date version of Flash, and if
// so, adds warning text to an element with id "flashwarning" if it exists on
// the page, and sets that element's class to "visible".
function showFlashWarningIfNecessary() {
  var warningNode = document.getElementById('flashwarning');
  if (!warningNode)
    return;
  if (!isFlashOutOfDate())
    return;
  
  var heading = document.createElement('h2');
  heading.textContent = 'For the best browsing experience, update Flash';
  warningNode.appendChild(heading);
  
  var warningText = document.createElement('p');

  var explanation = document.createTextNode(
      'You are running an out-of-date version of Flash Player, which may ' +
      'have security vulnerabilities or stability issues. Like all web ' +
      'browsers, Camino functions best when plug-ins such as Flash are ' +
      'up-to-date. We strongly recommend that you '); 
  warningText.appendChild(explanation);
  
  var updateLink = document.createElement('a');
  updateLink.textContent = 'update to the latest version of Flash';
  if (canRunLatestFlash()) {
    updateLink.setAttribute('href', 'http://get.adobe.com/flashplayer/');
  } else {
    updateLink.setAttribute(
        'href', 'http://get.adobe.com/flashplayer/otherversions/');
  }
  warningText.appendChild(updateLink);

  warningText.appendChild(document.createTextNode('.'));

  warningNode.appendChild(warningText);

  warningNode.className = 'visible';
}
